@@ -1,5 +1,4 @@
 import csv
-import heapq
 import logging
 import time
 from collections import defaultdict
@@ -8,10 +7,9 @@ import numpy as np
 
 from .ava_evaluation import object_detection_evaluation as det_eval
 from .ava_evaluation import standard_fields
-from .recall import eval_recalls
 
 
-def det2csv(dataset, results):
+def det2csv(dataset, results, custom_classes):
     csv_results = []
     for idx in range(len(dataset)):
         video_id = dataset.video_infos[idx]['video_id']
@@ -20,17 +18,21 @@ def det2csv(dataset, results):
         for label, _ in enumerate(result):
             for bbox in result[label]:
                 bbox_ = tuple(bbox.tolist())
+                if custom_classes is not None:
+                    actual_label = custom_classes[label + 1]
+                else:
+                    actual_label = label + 1
                 csv_results.append((
                     video_id,
                     timestamp,
-                ) + bbox_[:4] + (label + 1, ) + bbox_[4:])
+                ) + bbox_[:4] + (actual_label, ) + bbox_[4:])
     return csv_results
 
 
 # results is organized by class
-def results2csv(dataset, results, out_file):
+def results2csv(dataset, results, out_file, custom_classes=None):
     if isinstance(results[0], list):
-        csv_results = det2csv(dataset, results)
+        csv_results = det2csv(dataset, results, custom_classes)
 
     # save space for float
     def tostr(item):
@@ -45,7 +47,7 @@ def results2csv(dataset, results, out_file):
 
 
 def print_time(message, start):
-    print('==> %g seconds to %s' % (time.time() - start, message))
+    print('==> %g seconds to %s' % (time.time() - start, message), flush=True)
 
 
 def make_image_key(video_id, timestamp):
@@ -53,7 +55,7 @@ def make_image_key(video_id, timestamp):
     return f'{video_id},{int(timestamp):04d}'
 
 
-def read_csv(csv_file, class_whitelist=None, capacity=0):
+def read_csv(csv_file, class_whitelist=None):
     """Loads boxes and class labels from a CSV file in the AVA format.
 
     CSV file format described at https://research.google.com/ava/download.html.
@@ -62,8 +64,6 @@ def read_csv(csv_file, class_whitelist=None, capacity=0):
         csv_file: A file object.
         class_whitelist: If provided, boxes corresponding to (integer) class
         labels not in this set are skipped.
-        capacity: Maximum number of labeled boxes allowed for each example.
-        Default is 0 where there is no limit.
 
     Returns:
         boxes: A dictionary mapping each unique image key (string) to a list of
@@ -91,20 +91,16 @@ def read_csv(csv_file, class_whitelist=None, capacity=0):
         score = 1.0
         if len(row) == 8:
             score = float(row[7])
-        if capacity < 1 or len(entries[image_key]) < capacity:
-            heapq.heappush(entries[image_key],
-                           (score, action_id, y1, x1, y2, x2))
-        elif score > entries[image_key][0][0]:
-            heapq.heapreplace(entries[image_key],
-                              (score, action_id, y1, x1, y2, x2))
+
+        entries[image_key].append((score, action_id, y1, x1, y2, x2))
+
     for image_key in entries:
         # Evaluation API assumes boxes with descending scores
         entry = sorted(entries[image_key], key=lambda tup: -tup[0])
-        for item in entry:
-            score, action_id, y1, x1, y2, x2 = item
-            boxes[image_key].append([y1, x1, y2, x2])
-            labels[image_key].append(action_id)
-            scores[image_key].append(score)
+        boxes[image_key] = [x[2:] for x in entry]
+        labels[image_key] = [x[1] for x in entry]
+        scores[image_key] = [x[0] for x in entry]
+
     print_time('read file ' + csv_file.name, start)
     return boxes, labels, scores
 
@@ -162,15 +158,21 @@ def ava_eval(result_file,
              ann_file,
              exclude_file,
              max_dets=(100, ),
-             verbose=True):
+             verbose=True,
+             custom_classes=None):
 
     assert result_type in ['mAP']
 
     start = time.time()
     categories, class_whitelist = read_labelmap(open(label_file))
+    if custom_classes is not None:
+        custom_classes = custom_classes[1:]
+        assert set(custom_classes).issubset(set(class_whitelist))
+        class_whitelist = custom_classes
+        categories = [cat for cat in categories if cat['id'] in custom_classes]
 
     # loading gt, do not need gt score
-    gt_boxes, gt_labels, _ = read_csv(open(ann_file), class_whitelist, 0)
+    gt_boxes, gt_labels, _ = read_csv(open(ann_file), class_whitelist)
     if verbose:
         print_time('Reading detection results', start)
 
@@ -180,39 +182,10 @@ def ava_eval(result_file,
         excluded_keys = list()
 
     start = time.time()
-    boxes, labels, scores = read_csv(open(result_file), class_whitelist, 0)
+    boxes, labels, scores = read_csv(open(result_file), class_whitelist)
     if verbose:
         print_time('Reading detection results', start)
 
-    if result_type == 'proposal':
-        gts = [
-            np.array(gt_boxes[image_key], dtype=float)
-            for image_key in gt_boxes
-        ]
-        proposals = []
-        for image_key in gt_boxes:
-            if image_key in boxes:
-                proposals.append(
-                    np.concatenate(
-                        (np.array(boxes[image_key], dtype=float),
-                         np.array(scores[image_key], dtype=float)[:, None]),
-                        axis=1))
-            else:
-                # if no corresponding proposal, add a fake one
-                proposals.append(np.array([0, 0, 1, 1, 1]))
-
-        # Proposals used here are with scores
-        recalls = eval_recalls(gts, proposals, np.array(max_dets),
-                               np.arange(0.5, 0.96, 0.05))
-        ar = recalls.mean(axis=1)
-        ret = {}
-        for i, num in enumerate(max_dets):
-            print(f'Recall@0.5@{num}\t={recalls[i, 0]:.4f}')
-            print(f'AR@{num}\t={ar[i]:.4f}')
-            ret[f'Recall@0.5@{num}'] = recalls[i, 0]
-            ret[f'AR@{num}'] = ar[i]
-        return ret
-
     if result_type == 'mAP':
         pascal_evaluator = det_eval.PascalDetectionEvaluator(categories)
 
@@ -228,9 +201,7 @@ def ava_eval(result_file,
                     standard_fields.InputDataFields.groundtruth_boxes:
                     np.array(gt_boxes[image_key], dtype=float),
                     standard_fields.InputDataFields.groundtruth_classes:
-                    np.array(gt_labels[image_key], dtype=int),
-                    standard_fields.InputDataFields.groundtruth_difficult:
-                    np.zeros(len(gt_boxes[image_key]), dtype=bool)
+                    np.array(gt_labels[image_key], dtype=int)
                 })
         if verbose:
             print_time('Convert groundtruth', start)
