@@ -1,11 +1,12 @@
 import random
+import warnings
 from collections.abc import Sequence
 
 import mmcv
 import numpy as np
 from torch.nn.modules.utils import _pair
 
-from ..registry import PIPELINES
+from ..builder import PIPELINES
 
 
 def _init_lazy_if_proper(results, lazy):
@@ -42,6 +43,36 @@ def _init_lazy_if_proper(results, lazy):
         assert 'lazy' not in results, 'Use Fuse after lazy operations'
 
 
+@PIPELINES.register_module()
+class EntityBoxRescale:
+
+    def __init__(self, scale_factor):
+        raise NotImplementedError(
+            'This component should not be used in the '
+            'data pipeline and is removed in PR #782. Details see '
+            'https://github.com/open-mmlab/mmaction2/pull/782')
+
+
+@PIPELINES.register_module()
+class EntityBoxCrop:
+
+    def __init__(self, crop_bbox):
+        raise NotImplementedError(
+            'This component should not be used in the '
+            'data pipeline and is removed in PR #782. Details see '
+            'https://github.com/open-mmlab/mmaction2/pull/782')
+
+
+@PIPELINES.register_module()
+class EntityBoxFlip:
+
+    def __init__(self, img_shape):
+        raise NotImplementedError(
+            'This component should not be used in the '
+            'data pipeline and is removed in PR #782. Details see '
+            'https://github.com/open-mmlab/mmaction2/pull/782')
+
+
 @PIPELINES.register_module()
 class Imgaug:
     """Imgaug augmentation.
@@ -134,38 +165,59 @@ class Imgaug:
                 [self.imgaug_builder(t) for t in self.transforms])
 
     def default_transforms(self):
-        """Default transforms for imgaug."""
+        """Default transforms for imgaug.
+
+        Implement RandAugment by imgaug.
+        Plase visit `https://arxiv.org/abs/1909.13719` for more information.
+
+        Augmenters and hyper parameters are borrowed from the following repo:
+        https://github.com/tensorflow/tpu/blob/master/models/official/efficientnet/autoaugment.py # noqa
+
+        Miss one augmenter ``SolarizeAdd`` since imgaug doesn't support this.
+
+        Returns:
+            dict: The constructed RandAugment transforms.
+        """
+        # RandAugment hyper params
+        num_augmenters = 2
+        cur_magnitude, max_magnitude = 9, 10
+        cur_level = 1.0 * cur_magnitude / max_magnitude
 
         return [
-            dict(type='Rotate', rotate=(-30, 30)),
             dict(
                 type='SomeOf',
-                n=(0, 3),
+                n=num_augmenters,
                 children=[
                     dict(
-                        type='OneOf',
-                        children=[
-                            dict(type='GaussianBlur', sigma=(0, 0.5)),
-                            dict(type='AverageBlur', k=(2, 7)),
-                            dict(type='MedianBlur', k=(3, 11))
-                        ]),
+                        type='ShearX',
+                        shear=17.19 * cur_level * random.choice([-1, 1])),
                     dict(
-                        type='OneOf',
-                        children=[
-                            dict(
-                                type='Dropout', p=(0.01, 0.1),
-                                per_channel=0.5),
-                            dict(
-                                type='CoarseDropout',
-                                p=(0.03, 0.15),
-                                size_percent=(0.02, 0.05),
-                                per_channel=0.2),
-                        ]),
+                        type='ShearY',
+                        shear=17.19 * cur_level * random.choice([-1, 1])),
                     dict(
-                        type='AdditiveGaussianNoise',
-                        loc=0,
-                        scale=(0.0, 0.05 * 255),
-                        per_channel=0.5),
+                        type='TranslateX',
+                        percent=.2 * cur_level * random.choice([-1, 1])),
+                    dict(
+                        type='TranslateY',
+                        percent=.2 * cur_level * random.choice([-1, 1])),
+                    dict(
+                        type='Rotate',
+                        rotate=30 * cur_level * random.choice([-1, 1])),
+                    dict(type='Posterize', nb_bits=max(1, int(4 * cur_level))),
+                    dict(type='Solarize', threshold=256 * cur_level),
+                    dict(type='EnhanceColor', factor=1.8 * cur_level + .1),
+                    dict(type='EnhanceContrast', factor=1.8 * cur_level + .1),
+                    dict(
+                        type='EnhanceBrightness', factor=1.8 * cur_level + .1),
+                    dict(type='EnhanceSharpness', factor=1.8 * cur_level + .1),
+                    dict(type='Autocontrast', cutoff=0),
+                    dict(type='Equalize'),
+                    dict(type='Invert', p=1.),
+                    dict(
+                        type='Cutout',
+                        nb_iterations=1,
+                        size=0.2 * cur_level,
+                        squared=True),
                 ]),
         ]
 
@@ -188,7 +240,8 @@ class Imgaug:
 
         obj_type = args.pop('type')
         if mmcv.is_str(obj_type):
-            obj_cls = getattr(iaa, obj_type)
+            obj_cls = getattr(iaa, obj_type) if hasattr(iaa, obj_type) \
+                else getattr(iaa.pillike, obj_type)
         elif issubclass(obj_type, iaa.Augmenter):
             obj_cls = obj_type
         else:
@@ -323,6 +376,8 @@ class RandomScale:
     """
 
     def __init__(self, scales, mode='range', **kwargs):
+        warnings.warn('"RandomScale" is deprecated and will be removed in '
+                      'later versions. It is currently not used in MMAction2')
         self.mode = mode
         if self.mode not in ['range', 'value']:
             raise ValueError(f"mode should be 'range' or 'value', "
@@ -369,134 +424,6 @@ class RandomScale:
         return repr_str
 
 
-# Note, entity box transfroms are not added to: ThreeCrop, TenCrop,
-# MultiGroupCrop.
-@PIPELINES.register_module()
-class EntityBoxRescale:
-    """Rescale the entity box and proposals according to the image shape.
-
-    Required keys are "proposals", "gt_bboxes", added or modified keys are
-    "gt_bboxes". If original "proposals" is not None, "proposals" and
-    will be added or modified.
-
-    Args:
-        scale_factor (np.ndarray): The scale factor used entity_box rescaling.
-    """
-
-    def __init__(self, scale_factor):
-        self.scale_factor = scale_factor
-
-    def __call__(self, results):
-        scale_factor = np.concatenate([self.scale_factor, self.scale_factor])
-
-        proposals = results['proposals']
-        gt_bboxes = results['gt_bboxes']
-        results['gt_bboxes'] = gt_bboxes * scale_factor
-
-        if proposals is not None:
-            assert proposals.shape[1] == 4, (
-                'proposals shape should be in '
-                f'(n, 4), but got {proposals.shape}')
-            results['proposals'] = proposals * scale_factor
-
-        return results
-
-    def __repr__(self):
-        return f'{self.__class__.__name__}(scale_factor={self.scale_factor})'
-
-
-@PIPELINES.register_module()
-class EntityBoxCrop:
-    """Crop the entity boxes and proposals according to the cropped images.
-
-    Required keys are "proposals", "gt_bboxes", added or modified keys are
-    "gt_bboxes". If original "proposals" is not None, "proposals" will be
-    modified.
-
-    Args:
-        crop_bbox(np.ndarray | None): The bbox used to crop the original image.
-    """
-
-    def __init__(self, crop_bbox):
-        self.crop_bbox = crop_bbox
-
-    def __call__(self, results):
-        proposals = results['proposals']
-        gt_bboxes = results['gt_bboxes']
-
-        if self.crop_bbox is None:
-            return results
-
-        x1, y1, x2, y2 = self.crop_bbox
-        img_w, img_h = x2 - x1, y2 - y1
-
-        assert gt_bboxes.shape[-1] == 4
-        gt_bboxes_ = gt_bboxes.copy()
-        gt_bboxes_[..., 0::2] = np.clip(gt_bboxes[..., 0::2] - x1, 0,
-                                        img_w - 1)
-        gt_bboxes_[..., 1::2] = np.clip(gt_bboxes[..., 1::2] - y1, 0,
-                                        img_h - 1)
-        results['gt_bboxes'] = gt_bboxes_
-
-        if proposals is not None:
-            assert proposals.shape[-1] == 4
-            proposals_ = proposals.copy()
-            proposals_[..., 0::2] = np.clip(proposals[..., 0::2] - x1, 0,
-                                            img_w - 1)
-            proposals_[..., 1::2] = np.clip(proposals[..., 1::2] - y1, 0,
-                                            img_h - 1)
-            results['proposals'] = proposals_
-        return results
-
-    def __repr__(self):
-        return f'{self.__class__.__name__}(crop_bbox={self.crop_bbox})'
-
-
-@PIPELINES.register_module()
-class EntityBoxFlip:
-    """Flip the entity boxes and proposals with a probability.
-
-    Reverse the order of elements in the given bounding boxes and proposals
-    with a specific direction. The shape of them are preserved, but the
-    elements are reordered. Only the horizontal flip is supported (seems
-    vertical flipping makes no sense). Required keys are "proposals",
-    "gt_bboxes", added or modified keys are "gt_bboxes". If "proposals"
-    is not None, it will also be modified.
-
-    Args:
-        img_shape (tuple[int]): The img shape.
-    """
-
-    def __init__(self, img_shape):
-        self.img_shape = img_shape
-        assert mmcv.is_tuple_of(img_shape, int)
-
-    def __call__(self, results):
-        proposals = results['proposals']
-        gt_bboxes = results['gt_bboxes']
-        img_h, img_w = self.img_shape
-
-        assert gt_bboxes.shape[-1] == 4
-        gt_bboxes_ = gt_bboxes.copy()
-        gt_bboxes_[..., 0::4] = img_w - gt_bboxes[..., 2::4] - 1
-        gt_bboxes_[..., 2::4] = img_w - gt_bboxes[..., 0::4] - 1
-        if proposals is not None:
-            assert proposals.shape[-1] == 4
-            proposals_ = proposals.copy()
-            proposals_[..., 0::4] = img_w - proposals[..., 2::4] - 1
-            proposals_[..., 2::4] = img_w - proposals[..., 0::4] - 1
-        else:
-            proposals_ = None
-
-        results['proposals'] = proposals_
-        results['gt_bboxes'] = gt_bboxes_
-        return results
-
-    def __repr__(self):
-        repr_str = f'{self.__class__.__name__}(img_shape={self.img_shape})'
-        return repr_str
-
-
 @PIPELINES.register_module()
 class RandomCrop:
     """Vanilla square random crop that specifics the output size.
@@ -516,6 +443,37 @@ class RandomCrop:
         self.size = size
         self.lazy = lazy
 
+    def _box_crop(self, box, crop_bbox):
+        """Crop the bounding boxes according to the crop_bbox.
+
+        Args:
+            box (np.ndarray): The bounding boxes.
+            crop_bbox(np.ndarray): The bbox used to crop the original image.
+        """
+
+        x1, y1, x2, y2 = crop_bbox
+        img_w, img_h = x2 - x1, y2 - y1
+
+        box_ = box.copy()
+        box_[..., 0::2] = np.clip(box[..., 0::2] - x1, 0, img_w - 1)
+        box_[..., 1::2] = np.clip(box[..., 1::2] - y1, 0, img_h - 1)
+        return box_
+
+    def _all_box_crop(self, results, crop_bbox):
+        """Crop the gt_bboxes and proposals in results according to crop_bbox.
+
+        Args:
+            results (dict): All information about the sample, which contain
+                'gt_bboxes' and 'proposals' (optional).
+            crop_bbox(np.ndarray): The bbox used to crop the original image.
+        """
+        results['gt_bboxes'] = self._box_crop(results['gt_bboxes'], crop_bbox)
+        if 'proposals' in results and results['proposals'] is not None:
+            assert results['proposals'].shape[1] == 4
+            results['proposals'] = self._box_crop(results['proposals'],
+                                                  crop_bbox)
+        return results
+
     def __call__(self, results):
         """Performs the RandomCrop augmentation.
 
@@ -556,8 +514,9 @@ class RandomCrop:
 
         new_h, new_w = self.size, self.size
 
-        results['crop_bbox'] = np.array(
+        crop_bbox = np.array(
             [x_offset, y_offset, x_offset + new_w, y_offset + new_h])
+        results['crop_bbox'] = crop_bbox
 
         results['img_shape'] = (new_h, new_w)
 
@@ -586,8 +545,7 @@ class RandomCrop:
         # Process entity boxes
         if 'gt_bboxes' in results:
             assert not self.lazy
-            entity_box_crop = EntityBoxCrop(results['crop_bbox'])
-            results = entity_box_crop(results)
+            results = self._all_box_crop(results, results['crop_bbox'])
 
         return results
 
@@ -598,7 +556,7 @@ class RandomCrop:
 
 
 @PIPELINES.register_module()
-class RandomResizedCrop:
+class RandomResizedCrop(RandomCrop):
     """Random crop that specifics the area and height-weight ratio range.
 
     Required keys in results are "imgs", "img_shape", "crop_bbox" and "lazy",
@@ -738,8 +696,8 @@ class RandomResizedCrop:
 
         if 'gt_bboxes' in results:
             assert not self.lazy
-            entity_box_crop = EntityBoxCrop(results['crop_bbox'])
-            results = entity_box_crop(results)
+            results = self._all_box_crop(results, results['crop_bbox'])
+
         return results
 
     def __repr__(self):
@@ -751,11 +709,11 @@ class RandomResizedCrop:
 
 
 @PIPELINES.register_module()
-class MultiScaleCrop:
+class MultiScaleCrop(RandomCrop):
     """Crop images with a list of randomly selected scales.
 
     Randomly select the w and h scales from a list of scales. Scale of 1 means
-    the base size, which is the minimal of image weight and height. The scale
+    the base size, which is the minimal of image width and height. The scale
     level of w and h is controlled to be smaller than a certain value to
     prevent too large or small aspect ratio.
     Required keys are "imgs", "img_shape", added or modified keys are "imgs",
@@ -764,7 +722,7 @@ class MultiScaleCrop:
 
     Args:
         input_size (int | tuple[int]): (w, h) of network input.
-        scales (tuple[float]): Weight and height scales to be selected.
+        scales (tuple[float]): width and height scales to be selected.
         max_wh_scale_gap (int): Maximum gap of w and h scale levels.
             Default: 1.
         random_crop (bool): If set to True, the cropping bbox will be randomly
@@ -908,8 +866,7 @@ class MultiScaleCrop:
 
         if 'gt_bboxes' in results:
             assert not self.lazy
-            entity_box_crop = EntityBoxCrop(results['crop_bbox'])
-            results = entity_box_crop(results)
+            results = self._all_box_crop(results, results['crop_bbox'])
 
         return results
 
@@ -969,6 +926,17 @@ class Resize:
         self.interpolation = interpolation
         self.lazy = lazy
 
+    def _box_resize(self, box, scale_factor):
+        """Rescale the bounding boxes according to the scale_factor.
+
+        Args:
+            box (np.ndarray): The bounding boxes.
+            scale_factor (np.ndarray): The scale factor used for rescaling.
+        """
+        assert len(scale_factor) == 2
+        scale_factor = np.concatenate([scale_factor, scale_factor])
+        return box * scale_factor
+
     def __call__(self, results):
         """Performs the Resize augmentation.
 
@@ -1009,8 +977,12 @@ class Resize:
 
         if 'gt_bboxes' in results:
             assert not self.lazy
-            entity_box_rescale = EntityBoxRescale(self.scale_factor)
-            results = entity_box_rescale(results)
+            results['gt_bboxes'] = self._box_resize(results['gt_bboxes'],
+                                                    self.scale_factor)
+            if 'proposals' in results and results['proposals'] is not None:
+                assert results['proposals'].shape[1] == 4
+                results['proposals'] = self._box_resize(
+                    results['proposals'], self.scale_factor)
 
         return results
 
@@ -1091,18 +1063,37 @@ class Flip:
         flip_ratio (float): Probability of implementing flip. Default: 0.5.
         direction (str): Flip imgs horizontally or vertically. Options are
             "horizontal" | "vertical". Default: "horizontal".
+        flip_label_map (Dict[int, int] | None): Transform the label of the
+            flipped image with the specific label. Default: None.
         lazy (bool): Determine whether to apply lazy operation. Default: False.
     """
     _directions = ['horizontal', 'vertical']
 
-    def __init__(self, flip_ratio=0.5, direction='horizontal', lazy=False):
+    def __init__(self,
+                 flip_ratio=0.5,
+                 direction='horizontal',
+                 flip_label_map=None,
+                 lazy=False):
         if direction not in self._directions:
             raise ValueError(f'Direction {direction} is not supported. '
                              f'Currently support ones are {self._directions}')
         self.flip_ratio = flip_ratio
         self.direction = direction
+        self.flip_label_map = flip_label_map
         self.lazy = lazy
 
+    def _box_flip(self, box, img_width):
+        """Flip the bounding boxes given the width of the image.
+
+        Args:
+            box (np.ndarray): The bounding boxes.
+            img_width (int): The img width.
+        """
+        box_ = box.copy()
+        box_[..., 0::4] = img_width - box[..., 2::4]
+        box_[..., 2::4] = img_width - box[..., 0::4]
+        return box_
+
     def __call__(self, results):
         """Performs the Flip augmentation.
 
@@ -1120,6 +1111,10 @@ class Flip:
         results['flip'] = flip
         results['flip_direction'] = self.direction
 
+        if self.flip_label_map is not None and flip:
+            results['label'] = self.flip_label_map.get(results['label'],
+                                                       results['label'])
+
         if not self.lazy:
             if flip:
                 for i, img in enumerate(results['imgs']):
@@ -1142,8 +1137,12 @@ class Flip:
 
         if 'gt_bboxes' in results and flip:
             assert not self.lazy and self.direction == 'horizontal'
-            entity_box_flip = EntityBoxFlip(results['img_shape'])
-            results = entity_box_flip(results)
+            width = results['img_shape'][1]
+            results['gt_bboxes'] = self._box_flip(results['gt_bboxes'], width)
+            if 'proposals' in results and results['proposals'] is not None:
+                assert results['proposals'].shape[1] == 4
+                results['proposals'] = self._box_flip(results['proposals'],
+                                                      width)
 
         return results
 
@@ -1151,7 +1150,7 @@ class Flip:
         repr_str = (
             f'{self.__class__.__name__}('
             f'flip_ratio={self.flip_ratio}, direction={self.direction}, '
-            f'lazy={self.lazy})')
+            f'flip_label_map={self.flip_label_map}, lazy={self.lazy})')
         return repr_str
 
 
@@ -1424,7 +1423,7 @@ class ColorJitter:
 
 
 @PIPELINES.register_module()
-class CenterCrop:
+class CenterCrop(RandomCrop):
     """Crop the center area from images.
 
     Required keys are "imgs", "img_shape", added or modified keys are "imgs",
@@ -1506,8 +1505,7 @@ class CenterCrop:
 
         if 'gt_bboxes' in results:
             assert not self.lazy
-            entity_box_crop = EntityBoxCrop(results['crop_bbox'])
-            results = entity_box_crop(results)
+            results = self._all_box_crop(results, results['crop_bbox'])
 
         return results
 
@@ -1544,6 +1542,8 @@ class ThreeCrop:
                 to the next transform in pipeline.
         """
         _init_lazy_if_proper(results, False)
+        if 'gt_bboxes' in results or 'proposals' in results:
+            warnings.warn('ThreeCrop cannot process bounding boxes')
 
         imgs = results['imgs']
         img_h, img_w = results['imgs'][0].shape[:2]
@@ -1616,6 +1616,9 @@ class TenCrop:
         """
         _init_lazy_if_proper(results, False)
 
+        if 'gt_bboxes' in results or 'proposals' in results:
+            warnings.warn('TenCrop cannot process bounding boxes')
+
         imgs = results['imgs']
 
         img_h, img_w = results['imgs'][0].shape[:2]
@@ -1691,6 +1694,9 @@ class MultiGroupCrop:
             results (dict): The resulting dict to be modified and passed
                 to the next transform in pipeline.
         """
+        if 'gt_bboxes' in results or 'proposals' in results:
+            warnings.warn('MultiGroupCrop cannot process bounding boxes')
+
         imgs = results['imgs']
         img_h, img_w = imgs[0].shape[:2]
         crop_w, crop_h = self.crop_size
