@@ -1,6 +1,7 @@
 import copy
 import os.path as osp
 import warnings
+from collections import OrderedDict
 
 import mmcv
 import numpy as np
@@ -11,7 +12,7 @@ from ..localization import (eval_ap, load_localize_proposal_file,
                             perform_regression, temporal_iou, temporal_nms)
 from ..utils import get_root_logger
 from .base import BaseDataset
-from .registry import DATASETS
+from .builder import DATASETS
 
 
 class SSNInstance:
@@ -470,7 +471,7 @@ class SSNDataset(BaseDataset):
                                        for x in dets.tolist()])
             plain_detections[class_idx] = detection_list
 
-        eval_results = {}
+        eval_results = OrderedDict()
         for metric in metrics:
             if metric == 'mAP':
                 eval_dataset = metric_options.setdefault('mAP', {}).setdefault(
