@@ -7,10 +7,11 @@ from mmcv.utils import _BatchNorm
 from torch.nn.modules.utils import _ntuple, _triple
 
 from ...utils import get_root_logger
-from ..registry import BACKBONES
+from ..builder import BACKBONES
 
 try:
     from mmdet.models.builder import SHARED_HEADS as MMDET_SHARED_HEADS
+    from mmdet.models import BACKBONES as MMDET_BACKBONES
     mmdet_imported = True
 except (ImportError, ModuleNotFoundError):
     mmdet_imported = False
@@ -999,3 +1000,4 @@ class ResNet3dLayer(nn.Module):
 
 if mmdet_imported:
     MMDET_SHARED_HEADS.register_module()(ResNet3dLayer)
+    MMDET_BACKBONES.register_module()(ResNet3d)
