@@ -1,13 +1,11 @@
-import warnings
-
-from ..registry import BACKBONES
+from ..builder import BACKBONES
 from .resnet3d_slowfast import ResNet3dPathway
 
 try:
-    import mmdet  # noqa
     from mmdet.models.builder import BACKBONES as MMDET_BACKBONES
+    mmdet_imported = True
 except (ImportError, ModuleNotFoundError):
-    warnings.warn('Please install mmdet to use MMDET_BACKBONES')
+    mmdet_imported = False
 
 
 @BACKBONES.register_module()
@@ -50,5 +48,5 @@ class ResNet3dSlowOnly(ResNet3dPathway):
         assert not self.lateral
 
 
-if 'mmdet' in dir():
+if mmdet_imported:
     MMDET_BACKBONES.register_module()(ResNet3dSlowOnly)
