@@ -19,9 +19,9 @@ class AvgConsensus(nn.Module):
         super().__init__()
         self.dim = dim
 
-    def forward(self, input):
+    def forward(self, x):
         """Defines the computation performed at every call."""
-        return input.mean(dim=self.dim, keepdim=True)
+        return x.mean(dim=self.dim, keepdim=True)
 
 
 class BaseHead(nn.Module, metaclass=ABCMeta):
@@ -36,7 +36,7 @@ class BaseHead(nn.Module, metaclass=ABCMeta):
         num_classes (int): Number of classes to be classified.
         in_channels (int): Number of channels in input feature.
         loss_cls (dict): Config for building loss.
-            Default: dict(type='CrossEntropyLoss').
+            Default: dict(type='CrossEntropyLoss', loss_weight=1.0).
         multi_class (bool): Determines whether it is a multi-class
             recognition task. Default: False.
         label_smooth_eps (float): Epsilon used in label smooth.
@@ -46,7 +46,7 @@ class BaseHead(nn.Module, metaclass=ABCMeta):
     def __init__(self,
                  num_classes,
                  in_channels,
-                 loss_cls=dict(type='CrossEntropyLoss', loss_factor=1.0),
+                 loss_cls=dict(type='CrossEntropyLoss', loss_weight=1.0),
                  multi_class=False,
                  label_smooth_eps=0.0):
         super().__init__()
@@ -60,15 +60,13 @@ class BaseHead(nn.Module, metaclass=ABCMeta):
     def init_weights(self):
         """Initiate the parameters either from existing checkpoint or from
         scratch."""
-        pass
 
     @abstractmethod
     def forward(self, x):
         """Defines the computation performed at every call."""
-        pass
 
-    def loss(self, cls_score, labels):
-        """Calculate the loss given output ``cls_score`` and target ``labels``.
+    def loss(self, cls_score, labels, **kwargs):
+        """Calculate the loss given output ``cls_score``, target ``labels``.
 
         Args:
             cls_score (torch.Tensor): The output of the model.
@@ -81,8 +79,14 @@ class BaseHead(nn.Module, metaclass=ABCMeta):
         losses = dict()
         if labels.shape == torch.Size([]):
             labels = labels.unsqueeze(0)
+        elif labels.dim() == 1 and labels.size()[0] == self.num_classes \
+                and cls_score.size()[0] == 1:
+            # Fix a bug when training with soft labels and batch size is 1.
+            # When using soft labels, `labels` and `cls_socre` share the same
+            # shape.
+            labels = labels.unsqueeze(0)
 
-        if not self.multi_class:
+        if not self.multi_class and cls_score.size() != labels.size():
             top_k_acc = top_k_accuracy(cls_score.detach().cpu().numpy(),
                                        labels.detach().cpu().numpy(), (1, 5))
             losses['top1_acc'] = torch.tensor(
@@ -90,9 +94,15 @@ class BaseHead(nn.Module, metaclass=ABCMeta):
             losses['top5_acc'] = torch.tensor(
                 top_k_acc[1], device=cls_score.device)
 
-        elif self.label_smooth_eps != 0:
+        elif self.multi_class and self.label_smooth_eps != 0:
             labels = ((1 - self.label_smooth_eps) * labels +
                       self.label_smooth_eps / self.num_classes)
 
-        losses['loss_cls'] = self.loss_cls(cls_score, labels)
+        loss_cls = self.loss_cls(cls_score, labels, **kwargs)
+        # loss_cls may be dictionary or single tensor
+        if isinstance(loss_cls, dict):
+            losses.update(loss_cls)
+        else:
+            losses['loss_cls'] = loss_cls
+
         return losses
