@@ -5,8 +5,7 @@ import torch
 import torch.nn as nn
 
 from ...localization import temporal_iop, temporal_iou
-from ..builder import build_loss
-from ..registry import LOCALIZERS
+from ..builder import LOCALIZERS, build_loss
 from .base import BaseLocalizer
 from .utils import post_processing
 
@@ -291,8 +290,10 @@ class BMN(BaseLocalizer):
         for every_gt_bbox in gt_bbox:
             gt_iou_map = []
             for start, end in every_gt_bbox:
-                start = start.numpy()
-                end = end.numpy()
+                if isinstance(start, torch.Tensor):
+                    start = start.numpy()
+                if isinstance(end, torch.Tensor):
+                    end = end.numpy()
                 current_gt_iou_map = temporal_iou(self.match_map[:, 0],
                                                   self.match_map[:, 1], start,
                                                   end)
